@@ -137,6 +137,7 @@ static gboolean      process_mandir_pending    (YelpTocPager      *pager);
 #ifdef ENABLE_INFO
 static gboolean      process_info_pending      (YelpTocPager      *pager);
 #endif
+static gboolean      process_cleanup           (YelpTocPager      *pager);
 
 static void          toc_add_doc_info          (YelpTocPager      *pager,
 						YelpDocInfo       *doc_info);
@@ -364,6 +365,7 @@ toc_process_pending (YelpTocPager *pager)
 	process_info_pending,
 	process_xslt,
 #endif
+	process_cleanup,
 	NULL
     };
     static gint process_i = 0;
@@ -375,6 +377,8 @@ toc_process_pending (YelpTocPager *pager)
 
     readd = priv->pending_func(pager);
 
+    /* if the pending function returned FALSE, then we move on to
+     * calling the next function */
     if (!readd)
 	priv->pending_func = process_funcs[++process_i];
 
@@ -749,7 +753,6 @@ create_toc_from_index (YelpTocPager *pager, gchar *index_file)
 	    /* FIXME: need some error checking */
 	    mtime = (time_t) atoi ((gchar *)dirmtime);
 
-	    xmlFree (dirmtime);
 	    /* see if directory mtime has changed - if so recreate
 	     * the directory node */
 	    if (buf.st_mtime > mtime) {
@@ -801,12 +804,17 @@ create_toc_from_index (YelpTocPager *pager, gchar *index_file)
 		    priv->manpage_count++;
 		    xmlFree (manpage);
 	        }
-	    xmlXPathFreeObject (objmanpages);
+		xmlXPathFreeObject (objmanpages);
 	    }
+	    xmlFree (dirmtime);
 	    xmlFree (dirname);
 	    xmlXPathFreeObject (objdirmtime);
+	    xmlXPathFreeObject (objdirname);
 	}
 
+	/* cleanup */
+	xmlXPathFreeObject (objdirs);
+
 	if (priv->man_manhash) {
 	    g_hash_table_destroy (priv->man_manhash);
 	    priv->man_manhash = NULL;
@@ -839,6 +847,7 @@ process_mandir_pending (YelpTocPager *pager)
 	xmlXPathContextPtr xpath;
 	xmlXPathObjectPtr  obj;
 
+	/* NOTE: this document is free()'d at the end of the process_xslt function */
 	priv->toc_doc = xmlCtxtReadFile (priv->parser, DATADIR "/yelp/man.xml", NULL,
 					 XML_PARSE_NOBLANKS | XML_PARSE_NOCDATA  |
 					 XML_PARSE_NOENT    | XML_PARSE_NOERROR  |
@@ -1042,8 +1051,8 @@ process_info_pending (YelpTocPager *pager)
 	    continue;
 	} else {
 	    GIOChannel *channel;
-	    gchar *str;
-	    int len;
+	    gchar *str = NULL;
+	    gsize len;
 	    gchar ** files;
 	    gchar ** ptr;
 	    xmlNodePtr tmp;
@@ -1064,7 +1073,7 @@ process_info_pending (YelpTocPager *pager)
 	    }
 	    
 	    channel = yelp_io_channel_new_file (filename, NULL);
-	    g_io_channel_read_to_end (channel, &str, (gsize *) &len, NULL);
+	    g_io_channel_read_to_end (channel, &str, &len, NULL);
 	    g_io_channel_shutdown (channel, FALSE, NULL);
 	    
 	    files = g_strsplit (str, "\n", -1);
@@ -1322,7 +1331,12 @@ process_xslt (YelpTocPager *pager)
 
     if (!priv->toc_doc)
 	return FALSE;
-    priv->stylesheet = xsltParseStylesheetFile (BAD_CAST TOC_STYLESHEET);
+
+    /* only create and parse the stylesheet on the first call to this function */
+    if (!priv->stylesheet) {
+	priv->stylesheet = xsltParseStylesheetFile (BAD_CAST TOC_STYLESHEET);
+    }
+
     if (!priv->stylesheet) {
 	g_set_error (&error, YELP_ERROR, YELP_ERROR_PROC,
 		     _("The table of contents could not be processed. The "
@@ -1381,10 +1395,6 @@ process_xslt (YelpTocPager *pager)
 	xmlFreeDoc (priv->toc_doc);
 	priv->toc_doc = NULL;
     }
-    if (priv->stylesheet) {
-	xsltFreeStylesheet (priv->stylesheet);
-	priv->stylesheet = NULL;
-    }
     if (priv->transformContext) {
 	xsltFreeTransformContext (priv->transformContext);
 	priv->transformContext = NULL;
@@ -1393,6 +1403,21 @@ process_xslt (YelpTocPager *pager)
     return FALSE;
 }
 
+static gboolean
+process_cleanup (YelpTocPager *pager)
+{
+    YelpTocPagerPriv *priv = pager->priv;
+
+    /* cleanup the stylesheet used to process the toc */
+    if (priv->stylesheet) {
+	xsltFreeStylesheet (priv->stylesheet);
+	priv->stylesheet = NULL;
+    }
+
+    /* we only ever want to run this function once, so always return false */
+    return FALSE;
+}
+
 static void
 toc_add_doc_info (YelpTocPager *pager, YelpDocInfo *doc_info)
 {
